import os, glob, re
from setuptools import setup, find_packages


version = '0.4.0.2'

excludes = ['.git','.vscode']

archives = glob.glob("dist/*")
for archive in archives:
  os.remove(archive)

def not_excluded(path, excludes = []):
  for exclude in excludes:
    if exclude in path:
      return False
  return True

def fast_scandir(dirname,excludes=[]):
  subfolders = [f.path for f in os.scandir(dirname) if f.is_dir() and not_excluded(f.path, excludes)]
  for dirname in list(subfolders):
      subfolders.extend(fast_scandir(dirname))
  return subfolders

def get_files(dirs):
  files = []
  for dir in dirs:
    files.extend([x for x in glob.glob(dir + os.sep + '*') if not '.py' in x])
  return files

subfolders = fast_scandir('alphaz',excludes)
subfolders.append("alphaz")

data = {x.replace(os.sep,'.').replace('alphaz.',''):['*'] for x in subfolders}

files = get_files(subfolders)

with open("MANIFEST.in","w") as f:
  f.write("\n".join(["include %s"%x for x in files]))

setup(
  name = 'alphaz',
  packages = [x.replace(os.sep,'.') for x in subfolders],
  include_package_data=True,
  version = version,
  license='MIT',        
  description = 'A package full of very nice tools', 
  author = 'Aurèle',
  author_email="contact@aurele.eu",
  url="https://github.com/ZAurele/alphaz",
  download_url = 'https://github.com/ZAurele/alphaz/archive/refs/tags/%s.tar.gz'%version,
  keywords = ['Flask', 'Json'],
  install_requires=[       
            'vine==1.3.0',
            'werkzeug>=2.0.1',
            'numpy',
            'flask',
            'pandas',
            'pyyaml',
            'flask-mail',
            'h5py',
            'mysql-connector-python',
            'cx_Oracle',
            'pysocks',
            'pyjwt',
            'requests',
            'lxml',
            'configparser',
            'pymysql',
            'pysftp',
            'sqlalchemy==1.3.23',
            'requests-toolbelt',
            'marshmallow-sqlalchemy',
            'flask-marshmallow',
            'flask-sqlalchemy',
            'flask-admin',
            'flask-statistics',
            'flask-monitoringdashboard',
            'flask-debugtoolbar',
            'dicttoxml',
            'colorama',
            'gevent',
            'lxml',
            'sqlalchemy-utils',
            'concurrent_log_handler',
            'xmltodict',
            'flask-bcrypt>=0.7.1',
            'watchdog',
            'gitPython',
            'screenutils',
            'celery==4.4.7',
            'flower==0.9.7',
            'celery-message-consumer',
            'mkdocs',
            'mkdocs-material',
            'pymdown-extensions',
            'pygments',
            'openpyxl',
            'scp',
            'zeep',
            'gunicorn'
      ],
  classifiers=[
    'Development Status :: 3 - Alpha',      # Chose either "3 - Alpha", "4 - Beta" or "5 - Production/Stable" as the current state of your package
    'Intended Audience :: Developers', 
    'Topic :: Software Development :: Build Tools',
    'License :: OSI Approved :: MIT License', 
    'Programming Language :: Python :: 3', 
    'Programming Language :: Python :: 3.4',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
  ],
)
